@@ -123,10 +123,8 @@ sub query
 	confess unless $query = $oops->{queries}{$q};
 	$query =~ s/TP_/$oops->{table_prefix}/g;
 
-	local($@);
 	my $dbh = $args{dbh} || $oops->{dbh};
-	my $sth = eval { $dbh->prepare_cached($query, undef, 3) } || die $dbh->errstr;
-	die $@ if $@;
+	my $sth = $dbh->prepare_cached($query, undef, 3) || die $dbh->errstr;
 
 	if (exists $args{execute}) {
 		my @a = defined($args{execute})
@@ -144,11 +142,10 @@ sub query
 					$sth->bind_param($i+1, $a[$i]);
 				}
 			}
-			eval { $sth->execute() } || confess "could not execute '$query' with '@a':".$sth->errstr;
+			$sth->execute() || confess "could not execute '$query' with '@a':".$sth->errstr;
 		} else {
-			eval { $sth->execute(@a) } || confess "could not execute '$query' with '@a':".$sth->errstr;
+			$sth->execute(@a) || confess "could not execute '$query' with '@a':".$sth->errstr;
 		}
-		die $@ if $@;
 	}
 
 	return $sth;
@@ -200,7 +197,7 @@ sub post_new_object
 sub byebye
 {
 	my $oops = shift;
-	eval { $oops->{counterdbh}->disconnect() if $oops->{counterdbh} };
+	$oops->{counterdbh}->disconnect() if $oops->{counterdbh};
 }
 
 1;
